<?php
/* Simple MIME Mailer Class
 * Version 1.0.2
 * Copyright 2003-2005, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * DESCRIPTION
 *
 * Provides a very easy way to compose and send text or HTML E-mail messages
 * in MIME format, with or without file attachments and/or inline images.
 *
 *
 * CREDITS
 *
 * Includes code from the PHP manual, contributed by alex at bartl dot net.
 *
 *
 * EXAMPLE
 *
 *  // Mailer class usage example
 *  require_once('class_Mailer.php');
 *
 *  // instantiate the mailer and set our sender/recipient information
 *	$mailer = &new Mailer();
 *	$sender_email_address = 'me@example.com';
 *	$sender_name = 'Me';
 *	$recipient_email_address = 'you@example.com';
 *	$recipient_name = 'You';
 *	$subject = 'Test message';
 *
 *	// Compose the message
 *	$mailer->compose($sender_email_address,$recipient_email_address,$subject,$sender_name,$recipient_name);
 *
 *	// Set the text version of the message
 *	$message_text = "Hello,\n\nThis is a test.";
 *	$mailer->set_text($message_text);
 *
 *	// Set the HTML version of the message (optional)
 *	$message_html = "Hello,<br /><br /><strong>This is a test.</strong><br />";
 *	$mailer->set_html($message_html);
 *
 *	// Attach a file
 *	$mailer->attach('/tmp/myfile.bin');
 *	
 *	// Attach a picture (with mime type so that it shows up inline in rich mail clients)
 *	$mailer->attach('/tmp/mypicture.jpg','image/jpeg');
 *
 *	// Send the message
 *	$mailer->send()
 *
 *	// Done!
 *
 *
 * HISTORY
 *
 * 1.0.2 (11-Oct-2006)
 *
 *		- Corrected some mime issues when using file attachments
 *
 * 1.0.1 (18-Apr-2006)
 *
 *		- Added MAILER_RECIPIENT_OVERRIDE constant to override recipients for debugging
 *		- Added extra protection from header injection exploits
 *		- Added detect_hyperlinks(), text_to_html(), and html_to_text() functions to
 *		  automatically create an HTML message part from a text part or vice-versa.
 *		- Added plaintext_only class variable to optionally disable MIME and just send
 *		  a plaintext message.
 *
 *
 * LICENSE
 *
 * This script is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This script is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *	
 * You should have received a copy of the GNU General Public License along
 * with this script; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class Mailer {
	
	var $priority = 0;
	var $useragent = "eSite Media ScriptMailer";
	
	function Mailer($fromemail=NULL,$toemail=NULL,$subject=NULL,$fromname=NULL,$toname=NULL) {
		$this->related = false;
		$this->plaintext_only = false;
		$this->header_pairs = array();
		$this->priority_header_pairs = array();
		$this->compose($fromemail,$toemail,$subject,$fromname,$toname);
	}
	
	function _safe_email($email) {
		if (defined('MAILER_RECIPIENT_OVERRIDE') && strlen(MAILER_RECIPIENT_OVERRIDE)) {
			$email = MAILER_RECIPIENT_OVERRIDE;
			echo "[Mail override: $email]\n";
		}
		return $email;
	}
	
	function compose($fromemail=NULL,$toemail=NULL,$subject=NULL,$fromname=NULL,$toname=NULL) {
		$fromemail = $this->_oneline($fromemail);
		$toemail = $this->_safe_email($this->_oneline($toemail));
		$subject = $this->_oneline($subject);
		$fromname = $this->_oneline($fromname);
		$toname = $this->_oneline($toname);
		
		$this->sender = array(is_null($fromname)?$fromemail:$fromname,$fromemail);
		$this->recipient = array(is_null($toname)?trim($toemail):trim($toname),trim($toemail));
		$this->subject = trim($subject);
	}
	
	function carboncopy($toemail,$toname=NULL) {
		$toemail = $this->_safe_email($this->_oneline($toemail));
		$toname = $this->_oneline($toname);
		$this->cc[] = array($toemail,is_null($toname)?$toemail:$toname);
	}

	function blindcarboncopy($toemail,$toname=NULL) {
		$toemail = $this->_safe_email($this->_oneline($toemail));
		$toname = $this->_oneline($toname);
		$this->bcc[] = array($toemail,is_null($toname)?$toemail:$toname);
	}
	
	function set_text($text) {
		$this->text = $text;
	}
	
	function set_html($html) {
		$this->html = $html;
	}
	
	function clear_attachments() {
		$this->related = false;
		$this->attachments = array();
	}
	
	function attach($filename,$mimetype="application/octetstream",$contentid=false) {
		if ($contentid) $this->related = true;
		if (!is_string($contentid) /*&& !is_null($contentid)*/) {
			$contentid = md5(uniqid(rand(),1));
		}
		$this->attachments[] = array($filename,$mimetype,$contentid);
		
		return $contentid;
	}
	
	function get_headers() {
		$headers = "";
		foreach ($this->priority_header_pairs as $k=>$v) $headers .= "{$k}: {$v}\n";
		foreach ($this->header_pairs as $k=>$v) $headers .= "{$k}: {$v}\n";
		return $headers;
	}
	
	function set_header($name,$value) {
		$this->header_pairs[$name] = $value;
	}
	
	function set_priority_header($name,$value) {
		$this->priority_header_pairs[$name] = $value;
	}
	
	function build($for_php_mail = true) {
		$OB = "----=_NextPart_000_".md5(uniqid(rand(),1));
		$IB = "----=_NextPart_001_".md5(uniqid(rand(),1));
		$this->text = str_replace("\r\n","\n",$this->text);

		if (!strlen($this->html)) $this->html = preg_replace("/\n/","<br>\n",$this->text);
		if (!strlen($this->text)) $this->text = "Sorry, you need a mail client that supports HTML-format messages to read this E-mail.";
		
		if ($this->attachments && count($this->attachments)) $this->plaintext_only = false;
		
		$this->set_priority_header('From',$this->sender[0].' <'.$this->sender[1].'>');
		if (!$for_php_mail) {
			$this->set_priority_header('To',$this->recipient[0].' <'.$this->recipient[1].'>');
			$this->set_priority_header('Subject',$this->subject);
		}
		$this->set_priority_header('Reply-To',$this->sender[0].' <'.$this->sender[1].'>');
		$this->set_priority_header('Date',date('r'));
		
		if ($this->plaintext_only) {
			$this->set_priority_header('Content-Type','text/plain; charset=ISO-8859-1; format=flowed');
			$this->set_priority_header('Content-Transfer-Encoding','7bit');
		} else {
			$this->set_priority_header('MIME-Version','1.0');

			if ($this->related) {
				$contenttype = "multipart/related;\n\ttype=\"multipart/alternative\"";
			} elseif (count($this->attachments)) {
				$contenttype = "multipart/mixed";
			} else {
				$contenttype = "multipart/alternative";
			}
			//$contenttype = "multipart/alternative"; // test
			$this->set_priority_header('Content-Type',"$contenttype;\n\tboundary=\"".$OB."\"");
		}
		if ($this->priority) $this->set_priority_header('X-Priority',$this->priority);
		$this->set_priority_header('User-Agent',$this->useragent);
		
		$headers = $this->get_headers();
		
		if (is_array($this->cc)) {
			$cclist = "";
			foreach ($this->cc as $ccrecip) {
				list($ccemail,$ccname) = $ccrecip;
				if ($ccname) {
					$cclist .= ", \"$ccname\" <$ccemail>"; 
				} else {
					$cclist .= ", ".$ccemail;
				}
			}
			$headers .= "Cc: ".substr($cclist,2)."\n";
		}
		
		if (is_array($this->bcc)) {
			$cclist = "";
			foreach ($this->bcc as $ccrecip) {
				list($ccemail,$ccname) = $ccrecip;
				if ($ccname) {
					$cclist .= ", \"$ccname\" <$ccemail>"; 
				} else {
					$cclist .= ", ".$ccemail;
				}
			}
			$headers .= "Bcc: ".substr($cclist,2)."\n";
		}
		
		if ($this->plaintext_only) {
			$msg = $this->text;
		} else {
			//Messages start with text/html alternatives in OB
			$msg  = "This is a multi-part message in MIME format.\n";
			
			//plaintext section 
			$msg .= "\n--".$OB."\n";
			$msg .= "Content-Type: text/plain; charset=\"iso-8859-1\"\n";
			$msg .= "Content-Transfer-Encoding: quoted-printable\n\n";
			// plaintext goes here
			$msg .= $this->text."\n\n";
	
			$msg .= "\n--".$OB."\n";
	
			if ($this->attachments) {
				if ($this->related) {
					$msg .= "Content-Type: multipart/related;\n\tboundary=\"".$IB."\"\n\n";
				} else {
					$msg .= "Content-Type: multipart/alternative;\n\tboundary=\"".$IB."\"\n\n";
				}
		
				// html section 
				$msg .= "\n--".$IB."\n";
			}
			$msg .= "Content-Type: text/html; charset=\"iso-8859-1\"\n";
			$msg .= "Content-Transfer-Encoding: 7bit\n\n";
	
			// html goes here 
			//$msg .= chunk_split(base64_encode($this->html),76,"\n")."\n";
			$this->html = str_replace("\r\n","\n",$this->html);
			$html = explode("\n",$this->html);
			foreach ($html as $id=>$htmlline) {
					$msg .= wordwrap($htmlline)."\n";
			}
	//		$msg .= implode("",$html);
			
			// end of IB
	
			// attachments
			if ($this->attachments) {
				foreach ($this->attachments as $attachment) {
					list($attachment,$mimetype,$contentid) = $attachment;
					
					$msg .= "\n--".$IB."\n";
					$patharray = explode ("/", $attachment); 
					$filename=$patharray[count($patharray)-1];
					//test $msg.= "\n--".$OB."\n";
					$msg.="Content-Type: $mimetype;\n\tname=\"".$filename."\"\n";
					$msg.="Content-Transfer-Encoding: base64\n";
	//				$msg.="Content-Disposition: attachment;\n\tfilename=\"".$filename."\"\n";
					if ($contentid) {
						$msg.="Content-ID: <$contentid>\n";
						$msg.="Content-Disposition: inline;\n\tfilename=\"".$filename."\"\n";
					}
					$msg .= "\n";
					
					//file goes here
					$fd = fopen($attachment, "r");
					$filecontent = fread($fd,filesize($attachment));
					fclose($fd);
					$filecontent = chunk_split(base64_encode($filecontent),76,"\n");
					$msg .= $filecontent;
					$msg .= "\n";
				}
			}
			
			if ($this->attachments) {
				$msg .= "--".$IB."--\n";
			}
	
			$msg .= "\n--".$OB."--\n";
		}
		$this->msg = $msg;
		$this->headers = $headers;
		
		return true;
	}
	
	function send($smtphost = false,$smtpport = false,$smtpuser = false,$smtppass = false) {
		if (!$smtphost || !$smtpport) {
			$this->build();
			return mail(
				$this->recipient[1],
				$this->subject,
				$this->msg,
				$this->headers
			);
		} else {
			return $this->smtp_send($smtphost,$smtpport,$smtpuser,$smtppass);
		}
	}

	function pear_begin() {
		if (!isset($this->PEAR_PATH)) {
			if (file_exists('PEAR/Net/SMTP.php')) {
				$this->PEAR_PATH = 'PEAR/';
			} elseif (file_exists('Net/SMTP.php')) {
				$this->PEAR_PATH = '';
			} elseif (file_exists(dirname(__FILE__).'/PEAR/Net/SMTP.php')) {
				$this->PEAR_PATH = dirname(__FILE__).'/PEAR/';
			}
		}
		$path = getcwd();
		if ($path!=$this->PEAR_PATH) $this->PEAR_CWD = $path;
		
		chdir($this->PEAR_PATH);
	}
	
	function pear_end() {
		if ($this->PEAR_CWD) chdir($this->PEAR_CWD);
		$this->PEAR_CWD = false;
	}

	function smtp_send($smtphost,$smtpport,$smtpuser="",$smtppass="") {
		if (!$smtphost) return false;
		if (!$smtpport) $smtpport = 25;
		if (!$smtpuser) $smtpuser = "";
		if (!$smtppass) $smtppass = "";
		
		$this->build(true);
		$this->headers .= "Subject: ".$this->subject;
		
		$this->pear_begin();
		require_once($this->PEAR_PATH.'Net/SMTP.php');

		$hostname = function_exists('php_uname') ? php_uname('n') : $_SERVER['HTTP_HOST'];
		
		/* Create a new Net_SMTP object. */
		if (! ($smtp = new Net_SMTP($smtphost, $smtpport, $hostname))) {
		    return $this->_error("Unable to instantiate Net_SMTP object");
		}
		
		/* Connect to the SMTP server. */
		if (PEAR::isError($e = $smtp->connect())) {
		    return $this->_error($e->getMessage());
		}
		
		$smtp->setDebug(false);
		if ($smtpuser) {

			/* Authenticate */
			if (PEAR::isError($e = $smtp->auth($smtpuser,$smtppass))) {
			    return $this->_error($e->getMessage() . $smtp->debuglog);
			}			
		}
		
		/* Send the 'MAIL FROM:' SMTP command. */
		if (PEAR::isError($e = $smtp->mailFrom($this->sender[1]))) {
		    return $this->_error("Unable to set sender: " . $e->getMessage());
		}

		/* Address the message to each of the recipients. */
	    if (PEAR::isError($res = $smtp->rcptTo($this->recipient[1]))) {
	        return $this->_error("Unable to add recipient <$to>: " . $res->getMessage());
	    }
		
		/* Set the body of the message. */
		$message = $this->headers . "\n\n" . $this->msg;
		if (PEAR::isError($smtp->data($message))) {
		    return $this->_error("Unable to send message content");
		}
		
		/* Disconnect from the SMTP server. */
		$smtp->disconnect();
		$this->pear_end();

		return true;
	}	
	
	function _error($msg) {
		$this->pear_end();
		$this->error = $msg;
		return false;
	}
		
	// avoid potential mail header injections
	function _oneline($s) {
		if ( is_string($s) && (($p = strpos($s,"\n")) !== false) ) $s = substr($s,0,$p);
		return $s;
	}	
	
	function detect_hyperlinks($text) {
		// Thanks to various annotations in the PHP manual for these URL parsing regex's.
		$text = preg_replace(
			"/(?<!quot;|[=\"]|:\/\/)\b((\w+:\/\/|www\.|ftp\.).+?)".
			"(?=\W*([<>\s]|$))/i", "<a href=\"$1\" target=\"_blank\">$1</a>", $text);
		$text =  preg_replace("/href=\"www/i", "href=\"http://www", $text);
		
		$text = preg_replace(
			"/([\w-?&;#~=\.\/]+\@(\[?)[a-zA-Z0-9\-\.]+\.([a-zA-Z]{2,3}|[0-9]{1,3})(\]?))/i",
			"<A HREF=\"mailto:$1\">$1</A>",
			$text
		);
		
		/*
		$text = preg_replace(
			"/([a-z0-9_\.-]+\@[a-z0-9\-\.]+\.[a-z]{2,3})/i",
			"<A HREF=\"mailto:$1\">$1</A>",
			$text
		);
		*/		
		return $text;
	}
	
	function text_to_html($header="",$footer="") {
		$this->set_html( $header . $this->detect_hyperlinks(nl2br($this->text)) . $footer );
	}
	
	function html_to_text($extra_breaktags = false) {
		// strip out tags that contain text that we do not want in our E-mail
		$text = preg_replace('/<(script|style|title)[^\>]*?>[\s\S]*?<\/\\1>/i','',$this->html);
		
		// replace whitespace characters with spaces, as they're ignored in html
		$text = str_replace(array("\r","\n","\t","\v"),' ',$text);

		// set the tags that we want to insert newlines after
		$breaktags = array('br','/tr','/p','/div');
		
		// assemble our breaktags list and use it to process the HTML
		if (is_array($extra_breaktags)) $breaktags = array_merge($breaktags,$extra_breaktags);
		$breaktaglist = "";
		foreach ($breaktags as $k=>$tag) {
			$breaktaglist .= preg_quote($tag,'/') . '|';
		}
		$breaktaglist = substr($breaktaglist,0,strlen($breaktaglist)-1);
		$text = strip_tags(preg_replace('/<('.$breaktaglist.')(\s.*?)?>/i',"\n",$text));

		// decode all HTML entities
		$text = html_entity_decode($text);
		
		// replace runs of spaces with individual spaces, and remove leading/trailing spaces
		$text = preg_replace('/[\x20\xa0]{2,}/',' ',$text);
		$text = preg_replace('/^[\x20\xa0]+/m','',$text);
		$text = preg_replace('/[\x20\xa0]+$/m','',$text);

		// convert runs of 3 or more newlines into a double-newline
		$text = preg_replace('/\n{3,}/',"\n\n",$text);
		
		// done!
		$this->set_text( $text );
	}
		
}
?>